<?php

/**
 * Plugin Name: Discount payment Moona
 * Plugin URI: https://gitlab.com/moona1/woocommerce-moona-plugin
 * Description: Moona WooCommerce plugin allow to offer discount to your shoppers
 * Author: Moona
 * Author URI: https://www.moona.com
 * Version: 2.0.4
 * Text Domain: discount-payment-moona
 * Domain Path: /languages/
 * WC requires at least: 3.2.0
 * WC tested up to: 5.8
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
    // Exit if accessed directly
}

defined( 'WCMOONA_ECOMMERCE_SOLUTION' ) || define( 'WCMOONA_ECOMMERCE_SOLUTION', 'WooCommerce' );
defined( 'WCMOONA_PLUGIN_VERSION' ) || define( 'WCMOONA_PLUGIN_VERSION', 'WC_2.0.4' );
defined( 'WCMOONA_API_VERSION' ) || define( 'WCMOONA_API_VERSION', '1.0.0' );
defined( 'WCMOONA_BASE_PATH' ) || define( 'WCMOONA_BASE_PATH', realpath( dirname( __FILE__ ) ) );
defined( 'WCMOONA_BASE_URL' ) || define( 'WCMOONA_BASE_URL', plugin_dir_url( __FILE__ ) );
defined( 'WCMOONA_BASE_NAME' ) || define( 'WCMOONA_BASE_NAME', basename( dirname( __FILE__ ) ) ); // dirname( plugin_basename( __FILE__ ) ) . '/'
defined( 'WCMOONA_FILE_VERSION' ) || define( 'WCMOONA_FILE_VERSION', '2.0.4' );

defined( 'MOONA_API_ENDPOINT' ) || define( 'MOONA_API_ENDPOINT', 'https://api.moona.com/' );
defined( 'MOONA_API_STAGING_ENDPOINT' ) || define( 'MOONA_API_STAGING_ENDPOINT', 'https://staging.api.moona.com/' );

if ( ! class_exists( 'WC_Moona' ) ) {

    /**
     * Class WC_Moona
     */
    class WC_Moona {

        public static $log = false ;

        private $moona_options;
        private $banners = array();
        private $display_discount;

        private static $banner_html = '<p style="font-family:aeonikregular;font-size:20px;color:#292B36;padding:10px;background-color:#f5f6fa;text-align:center;"><strong><span style="font-size:25px;">£5</span> discount each time you pay with </strong><img style="border-radius:0px;vertical-align: baseline; display:unset; width:103px; height:17px;" src="'. WCMOONA_BASE_URL . 'assets/images/logo-moona-vectorized.svg" alt="Moona" ></img></p>';
        private static $banner_post_order_html = '<p style="font-family:aeonikregular;font-size:20px;color:#292B36;padding:10px;background-color:#f5f6fa;text-align:center;"><strong><span style="font-size:25px;">£5</span> off on each order with <img style="border-radius:0px;vertical-align:baseline;display:unset;width:103px; height:17px;" src="'. WCMOONA_BASE_URL . 'assets/images/logo-moona-vectorized.svg" alt="Moona" ></img><strong></p>';
        private static $banner_above_cta_text_html = '<p style="font-family:aeonikregular;font-size:20px;color:#292B36;padding:10px;"><strong>£5 OFF by paying with Moona</strong></p>';
        private static $banner_center_above_cta_html = '<p style="font-family:aeonikregular;font-size:20px;color:#292B36;padding:10px;text-align:center;"><strong>£5 OFF by paying with </strong><img style="border-radius:0px;vertical-align:baseline;display:unset;width:85px;height:14px;" src="'. WCMOONA_BASE_URL . 'assets/images/logo-moona-vectorized.svg" alt="Moona" ></img></p>';
        private static $banner_square_full_img = '<img style="border-radius:0px;width:100%;" src="'. WCMOONA_BASE_URL . 'assets/images/banner-square-full.png" alt="Moona"></img>';

        /**
		 * The reference the *Singleton* instance of this class.
		 *
		 * @var $instance
		 */
        private static $instance = null;

        /**
		 * Returns the *Singleton* instance of this class.
		 *
		 * @return self::$instance The *Singleton* instance.
		 */
        public static function get_instance()
        {
			if (is_null(self::$instance)) {
				self::$instance = new self;
			}
			return self::$instance;
        }

        /**
		 * Private clone method to prevent cloning of the instance of the
		 * *Singleton* instance.
		 *
		 * @return void
		 */
		private function __clone() {
		}

		/**
		 * Private unserialize method to prevent unserializing of the *Singleton*
		 * instance.
		 *
		 * @return void
		 */
		private function __wakeup() {
        }
        
        /**
         * Constructor
         */
        public function __construct() {
            // Load Amplitude
            require_once WCMOONA_BASE_PATH . '/includes/moona_amplitude.php';

            // Check WooCommerce is active
            if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) || in_array( 'woocommerce/woocommerce.php', array_keys( get_site_option( 'active_sitewide_plugins' ) ) ) ) {
                
                // Load Moona plugin
                add_action( 'plugins_loaded', array( $this, 'wcmoona_plugin_to_load' ), 0 );

                // Add link to the Moona settings
                add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), array( $this, 'add_action_settings_link' ) );

                // Amplitude : Checkout view
                add_action( 'woocommerce_after_checkout_form', array( $this, 'after_checkout_form_loaded'), 10, 1 );

                // Amplitude : Click CTA
                add_action( 'woocommerce_checkout_order_processed', array( $this, 'action_checkout_order_processed'), 10, 1 );

                // Display discount at checkout
                add_action( 'woocommerce_review_order_before_order_total', array( $this, 'review_order_before_order_total_callback' ), 10, 0 );
                add_action( 'woocommerce_review_order_after_order_total', array( $this, 'review_order_after_order_total_callback' ), 10, 0 );
                add_filter( 'woocommerce_cart_total', array( $this, 'linethrough_price' ), 10, 1 );
                add_filter( 'woocommerce_get_order_item_totals',  array( $this, 'display_moona_discount_new_row' ), 10, 2 );
                add_filter( 'woocommerce_get_formatted_order_total',  array( $this, 'display_moona_order_total' ), 10, 4 );
                
                // Order status changed
                add_action( 'woocommerce_order_status_changed', array( $this, 'action_order_status_changed'), 10, 4 );
                // Other hook ways
                // woocommerce_payment_complete
                // woocommerce_new_order
                // woocommerce_order_status_* => woocommerce_order_status_completed
                // woocommerce_order_status_pending
                // woocommerce_order_status_failed
                // woocommerce_order_status_on-hold
                // woocommerce_order_status_processing
                // woocommerce_order_status_completed
                // woocommerce_order_status_refunded
                // woocommerce_order_status_cancelled
                add_action( 'woocommerce_new_order', array( $this, 'action_create_moona_order'), 10, 2 );
                
                // Init Cron job & add new 'every 10 minutes' schedule'
                // Schedule an event unless already scheduled
                add_action('wp', array( $this, 'create_moona_cron_jobs') );
                // Define a cron job interval if it doesn't exist
                add_filter( 'cron_schedules', array( $this, 'cron_every_ten_minutes') );
                add_action('moona_cron', array( $this, 'update_orders') );

                add_action('init', array( $this, 'update_plugin_version' ), 10, 0 );

                // Save Moona position on Payments tabs > Save settings
                add_action( "woocommerce_update_options_checkout", array( $this, 'action_woocommerce_settings_save_checkout' ), 10, 1 ); 
            }

            add_action( 'wp_enqueue_scripts', array( $this, 'wp_enqueue_scripts' ) );

            // Customize banners
            $this->moona_options = get_option('woocommerce_discount-payment-moona_settings');
            $this->customize_banners();

            // Banners
            add_action( 'woocommerce_before_shop_loop', array( $this, 'action_woocommerce_before_shop_loop' ), 10, 2 ); 
            add_action( 'woocommerce_before_single_product', array( $this, 'action_woocommerce_before_single_product' ), 10, 2 ); 
            add_action( 'woocommerce_before_cart_table', array( $this, 'action_woocommerce_before_cart_table' ), 10, 2 ); 
            add_action( 'woocommerce_before_checkout_form', array( $this, 'action_woocommerce_before_checkout_form' ), 10, 2 ); 
            add_action( 'woocommerce_thankyou', array( $this, 'action_woocommerce_thankyou' ), 10, 1 );

            add_action( 'woocommerce_before_add_to_cart_form', array( $this, 'action_woocommerce_before_add_to_cart_form' ), 10, 2 );
            add_action( 'woocommerce_proceed_to_checkout', array( $this, 'action_woocommerce_proceed_to_checkout' ), 10, 2 ); 
            add_action( 'woocommerce_review_order_before_payment', array( $this, 'action_woocommerce_review_order_before_payment' ), 10, 2 );
            add_action( 'woocommerce_mini_cart_contents', array( $this, 'action_woocommerce_mini_cart_contents' ), 10, 2 ); 

            add_action( 'woocommerce_after_mini_cart', array( $this, 'action_woocommerce_after_mini_cart' ), 10, 2 ); 
        }

        public function display_moona_order_total( $formatted_total, $order, $tax_display, $display_refunded ) {

            // Display discount on Moona discount for order-pay and order-received
            $displayDiscount = $order->get_meta('_has_moona_discount');
            // Display discount if &disc=1
            $displayDiscountGet = (isset($_GET['disc']) && $_GET['disc'] == 1) ? true : false;
            
            if (
                (is_wc_endpoint_url('order-pay') && $order->get_payment_method() === 'discount-payment-moona') ||
                (is_wc_endpoint_url('order-received') && ($displayDiscountGet || $displayDiscount === 'true' || $order->get_payment_method() === 'discount-payment-moona'))
             ) {
                $discount = (empty($order->get_meta('_moona_discount_amount'))) ? 500 : intval($order->get_meta('_moona_discount_amount'));
                $formatted_total = '<span style="text-decoration:line-through;">'. $formatted_total .'</span> ';
                $new_total = wc_price( $order->get_total() - ($discount / 100), array( 'currency' => $order->get_currency() ) );
                $new_total = str_replace('class="woocommerce-Price-amount amount"', 'class="woocommerce-Price-amount amount" style="color:red; font-weight: 600;"', $new_total);
                $formatted_total .= $new_total;
            }

            return $formatted_total;
        }

        function wp_enqueue_scripts() {

            $this->moona_options = get_option('woocommerce_discount-payment-moona_settings');
            if ( $this->moona_options === false ) {
                return;
            }

            // Load css file
            wp_enqueue_style('wc_moona_css', WCMOONA_BASE_URL . 'assets/css/moona.css', array(), WCMOONA_FILE_VERSION);

            if ( is_checkout() || is_wc_endpoint_url('order-pay') ) {
                // Allow to call the script moona.js to update the checkout text of Moona
                wp_enqueue_script('moona-js', WCMOONA_BASE_URL . 'assets/js/moona.js', array( 'jquery' ), WCMOONA_FILE_VERSION, false );

                // Add JS variable to allow to send POST AJAX message
                wp_localize_script( 'moona-js', 'ajax_object', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
            }
        }

        function display_moona_discount_new_row( $total_rows, $order ) {
            
            $displayDiscount = $order->get_meta('_has_moona_discount');
            if ( $order->get_payment_method() === 'discount-payment-moona' || 
                ($order->get_payment_method() === 'discount-payment-main-moona' && $displayDiscount === 'true') ) {

                // The altered code for the thankyou page
                if ( is_wc_endpoint_url('order-received') ) {
                    $source = 'order-summary';
                } 
                // Only on emails notifications
                else if ( ! is_wc_endpoint_url() || ! is_admin() ) {
                    $source = 'order-email';
                }
                
                $this->display_discount = $this->get_discount_text($order->get_total(), $source);

                if ($this->display_discount && $this->display_discount['display_mode'] === 'text_with_discount' && $this->display_discount['display_with_discount']) {
                    // No need anymore because it's already made with display_moona_order_total function
                    // if( isset($total_rows['order_total']) ) {
                    //     $total_rows['order_total']['value'] = '<span style="text-decoration:line-through">' . $total_rows['order_total']['value'] . '</span>';
                    // }
        
                    $total_rows['moona_discount'] = array(
                        'label' => $this->display_discount['text_with_discount'],
                        'value' => $this->display_discount['total_with_discount']
                    );
                }
            }
            
            return $total_rows;
        }

        function review_order_before_order_total_callback() {

            $chosen_payment_method = WC()->session->get('chosen_payment_method');
            if ( $this->moona_options === false || $chosen_payment_method !== 'discount-payment-moona') {
                return;
            }

            $this->display_discount = $this->get_discount_text(WC()->cart->total, 'checkout');
        }

        function get_discount_text($total, $source) {
            $body = array(
                'ecommerce_solution' => WCMOONA_ECOMMERCE_SOLUTION,
                'ecommerce_version' => ( defined( 'WC_VERSION' ) ) ? WC_VERSION : '',
                'plugin_version' => WCMOONA_PLUGIN_VERSION,
                'api_version' => WCMOONA_API_VERSION,
                'lang' => get_locale(),
                'price' => $total * 100,
                'currency_code' => get_woocommerce_currency(),
                'currency_symbol' => (htmlentities(get_woocommerce_currency_symbol()) === '&amp;pound;') ? '£' : get_woocommerce_currency_symbol(),
                'source' => $source
            );

            $url = ( $this->moona_options['test_mode'] === 'yes' ) ? MOONA_API_STAGING_ENDPOINT : MOONA_API_ENDPOINT;
            $url .= 'ecommerce/display_discount';
            $response = wp_remote_post( $url, array(
                    'method' => 'POST',
                    'timeout' => 20,
                    'headers' => array(
                        'Content-Type' => 'application/json; charset=utf-8',
                        'x-api-key' => ( $this->moona_options['test_mode'] === 'yes' ) ? $this->moona_options['moona_sk_test'] : $this->moona_options['moona_sk_live']
                    ),
                    'body' => json_encode($body)
                )
            );

            if ( is_wp_error( $response ) || wp_remote_retrieve_response_code( $response ) !== 200 ) {
                $this->log( 'error on get_discount_text() process' );
            } else {
                // {
                //     display_mode: 'text_with_discount', // or text_without_discount => allow to know if we display the text_with_discount or the other one
                //     display_with_discount: true, 
                //     text_with_discount: '<span style="color:red;">New Total With Moona</span>', 
                //     total_with_discount: `<span style="color:red;">${currency_symbol} ${new_total}</span>`,
                //     display_without_discount: true,
                //     text_without_discount: '<span style="color:red;">£5 off with Moona on orders above £10</span>' // Add items worth £x, and get £5 off with
                // }
                return json_decode( $response['body'], true );
            }
        }

        function linethrough_price($wc_price) {

            $chosen_payment_method = WC()->session->get('chosen_payment_method');
            if ( is_checkout() 
                    && $this->display_discount
                    && $chosen_payment_method === 'discount-payment-moona' 
                    && $this->display_discount['display_mode'] === 'text_with_discount' 
                    && $this->display_discount['display_with_discount']) {
                return '<span style="text-decoration:line-through">'. $wc_price .'</span>';
            }
            return $wc_price;
        }

        function action_woocommerce_settings_save_checkout( $array ) {
            $url = wc_get_current_admin_url();
            if (strpos($url, 'tab=checkout') !== false && strpos($url, 'section=') === false) {
                $GLOBALS['wc_moona_amplitude']->saveMoonaPosition();
            }
        }

        function review_order_after_order_total_callback() {

            $chosen_payment_method = WC()->session->get('chosen_payment_method');
            if (!$this->display_discount || !( is_checkout() && $chosen_payment_method === 'discount-payment-moona' )) {
                return;
            }
            
            if ($this->display_discount['display_mode'] === 'text_with_discount' && $this->display_discount['display_with_discount']) {
                echo '<tr class="order-total"><th>'. $this->display_discount['text_with_discount'] .'</th><th>'. $this->display_discount['total_with_discount'] .'</th></tr>';
            } 
            else if ($this->display_discount['display_mode'] === 'text_without_discount' && $this->display_discount['text_without_discount']) {
                echo '<tr class="order-total"><th colspan="2" >'. $this->display_discount['text_without_discount'] .'</th></tr>';
            }
        }

        /**
         * Customize banners according to WS
         */
        function customize_banners() {

            $this->moona_options = get_option('woocommerce_discount-payment-moona_settings');
            if ( $this->moona_options === false ) {
                return;
            }

            // Update banners every 24h
            if ( isset($this->moona_options['_timestamp_update_banners']) ) {
                if ( ($this->moona_options['_timestamp_update_banners'] + 24*60*60) > time() ) {
                    return;
                }
            }
            $options = $this->moona_options;
            $options['_timestamp_update_banners'] = time();
            
            // Fetch the banners
            $locale = get_locale();
            $body = array(
                'ecommerce_solution' => WCMOONA_ECOMMERCE_SOLUTION,
                'ecommerce_version' => ( defined( 'WC_VERSION' ) ) ? WC_VERSION : '',
                'plugin_version' => WCMOONA_PLUGIN_VERSION,
                'api_version' => WCMOONA_API_VERSION,
                'lang' => $locale
            );

            $url = ( $this->moona_options['test_mode'] === 'yes' ) ? MOONA_API_STAGING_ENDPOINT : MOONA_API_ENDPOINT;
            $url .= 'ecommerce/banners';
            $response = wp_remote_post( $url, array(
                    'method' => 'POST',
                    'timeout' => 20,
                    'headers' => array(
                        'Content-Type' => 'application/json; charset=utf-8',
                        'x-api-key' => ( $this->moona_options['test_mode'] === 'yes' ) ? $this->moona_options['moona_sk_test'] : $this->moona_options['moona_sk_live']
                    ),
                    'body' => json_encode($body)
                )
            );

            if ( is_wp_error( $response ) || wp_remote_retrieve_response_code( $response ) !== 200 ) {
                $this->log( 'error on customize_banners() process : ' . print_r($response, true) );
            } else {
                $body = json_decode( $response['body'], true );
                $this->banners = $body['banners'];

                // Save the banners
                $options['_banner_top_shop_archives_html'] = $this->banners['banner_top_shop_archives'];
                $options['_banner_top_product_page_html'] = $this->banners['banner_top_product_page'];
                $options['_banner_top_cart_page_html'] = $this->banners['banner_top_cart_page'];
                $options['_banner_top_checkout_page_html'] = $this->banners['banner_top_checkout_page'];
                $options['_banner_top_thankyou_page_html'] = $this->banners['banner_top_thankyou_page'];
                $options['_banner_above_cta_product_page_html'] = $this->banners['banner_above_cta_product_page'];
                $options['_banner_above_cta_cart_page_html'] = $this->banners['banner_above_cta_cart_page'];
                $options['_banner_above_cta_checkout_page_html'] = $this->banners['banner_above_cta_checkout_page'];
                $options['_banner_above_cta_minicart_html'] = $this->banners['banner_above_cta_minicart'];
                $options['_banner_square_cart_page_html'] = $this->banners['banner_square_cart_page'];
            }
            update_option('woocommerce_discount-payment-moona_settings', $options);
        }

        // V1.0.0 : Retro-compatibility
        // Update the merchant plugin version
        function update_plugin_version() {

            $this->moona_options = get_option('woocommerce_discount-payment-moona_settings');
            if ( $this->moona_options === false ) {
                return;
            }

            // Check the version
            if ( isset($this->moona_options['_merchant_plugin_version']) ) {
                if ( $this->moona_options['_merchant_plugin_version'] === WCMOONA_PLUGIN_VERSION ) {
                    return;
                }
            }
            $options = $this->moona_options;
            $options['_merchant_plugin_version'] = WCMOONA_PLUGIN_VERSION;
            update_option('woocommerce_discount-payment-moona_settings', $options);

            wcmoona_plugin_status('updated');
        }

        // To know if moona is activated
        function is_moona_activated() {
            $this->moona_options = get_option('woocommerce_discount-payment-moona_settings');
            if ( $this->moona_options === false ) {
                return false;
            }
            if ( $this->moona_options['enabled'] !== 'yes' ) {
                return false;
            }
            if ( !in_array( get_woocommerce_currency(), array( 'GBP' ) ) ) {
                return false;
            }
            return true;
        }

        function action_woocommerce_before_shop_loop() {
            if (isset($this->moona_options['banner_top_shop_archives']) && $this->moona_options['banner_top_shop_archives'] === 'yes' && $this->is_moona_activated()) {
                if (isset($this->moona_options['_banner_top_shop_archives_html'])) {
                    echo $this->moona_options['_banner_top_shop_archives_html'];
                } else {
                    echo self::$banner_html;
                }
            }
        }

        function action_woocommerce_before_single_product() {
            if (isset($this->moona_options['banner_top_product_page']) && $this->moona_options['banner_top_product_page'] === 'yes' && $this->is_moona_activated()) {
                if (isset($this->moona_options['_banner_top_product_page_html'])) {
                    echo $this->moona_options['_banner_top_product_page_html'];
                } else {
                    echo self::$banner_html;
                }
            }
        }

        function action_woocommerce_before_cart_table() {
            if (isset($this->moona_options['banner_top_cart_page']) && $this->moona_options['banner_top_cart_page'] === 'yes' && $this->is_moona_activated()) {
                if (isset($this->moona_options['_banner_top_cart_page_html'])) {
                    echo $this->moona_options['_banner_top_cart_page_html'];
                } else {
                    echo self::$banner_html;
                }
            }
        }

        function action_woocommerce_before_checkout_form() {
            if (isset($this->moona_options['banner_top_checkout_page']) && $this->moona_options['banner_top_checkout_page'] === 'yes' && $this->is_moona_activated()) {
                if (isset($this->moona_options['_banner_top_checkout_page_html'])) {
                    echo $this->moona_options['_banner_top_checkout_page_html'];
                } else {
                    echo self::$banner_html;
                }
            }
        }

        function action_woocommerce_thankyou( $order_id ) {
            if (isset($this->moona_options['banner_top_thankyou_page']) && $this->moona_options['banner_top_thankyou_page'] === 'yes' && $this->is_moona_activated()) {
                if (isset($this->moona_options['_banner_top_thankyou_page_html'])) {
                    echo $this->moona_options['_banner_top_thankyou_page_html'];
                } else {
                    echo self::$banner_post_order_html;
                }
            }

            // Order confirmation
            // When the shopper see the order confirmation page
            // Amplitude event
            if ( ! $order_id ) {
                return;
            }

            $order = wc_get_order( $order_id );            
            $shortVersion = explode(".", WC_VERSION);
            
            $paymentMethodID = $order->get_payment_method();
            if ($paymentMethodID === 'discount-payment-moona') {
                $paymentMethodID = 'moona-discount';
            } else if ($paymentMethodID === 'discount-payment-main-moona') {
                $paymentMethodID = 'moona-payment';
            }

            $eventProperties = array(
                'merchantSite' =>  get_bloginfo( 'name' ),
                'merchantURL' => get_permalink( wc_get_page_id( 'shop' ) ),
                'orderID' => $order_id,
                'currency' => get_woocommerce_currency(),
                'currencySymbol' => (htmlentities(get_woocommerce_currency_symbol()) === '&amp;pound;') ? '£' : get_woocommerce_currency_symbol(),
                'orderTotalValue' => $order->get_total(),
                'paymentMethodID' => $paymentMethodID,
                'paymentMethodLabel' => $order->get_payment_method_title(),
                'orderStatus' => $order->get_status(),
                'ecommerceSolution' => WCMOONA_ECOMMERCE_SOLUTION,
                'ecommerceVersion' => WC_VERSION, // ex : 5.0.0
                'ecommerceShortVersion' => $shortVersion[0].'.'.$shortVersion[1], // ex : 5.0
                'pluginVersion' => WCMOONA_PLUGIN_VERSION
            );

            if (isset($this->moona_options['merchant_id'])) {
                $eventProperties['merchantID'] = $this->moona_options['merchant_id'];
            }

            $GLOBALS['wc_moona_amplitude']->logEvent('View - Order confirmation', $eventProperties);
        }

        function action_woocommerce_before_add_to_cart_form() {
            if (isset($this->moona_options['banner_above_cta_product_page']) && $this->moona_options['banner_above_cta_product_page'] === 'yes' && $this->is_moona_activated()) {
                if (isset($this->moona_options['_banner_above_cta_product_page_html'])) {
                    echo $this->moona_options['_banner_above_cta_product_page_html'];
                } else {
                    echo self::$banner_above_cta_text_html;
                }
            }
        }

        function action_woocommerce_proceed_to_checkout() {
            if (isset($this->moona_options['banner_above_cta_cart_page']) && $this->moona_options['banner_above_cta_cart_page'] === 'yes' && $this->is_moona_activated()) {
                if (isset($this->moona_options['_banner_above_cta_cart_page_html'])) {
                    echo $this->moona_options['_banner_above_cta_cart_page_html'];
                } else {
                    echo self::$banner_center_above_cta_html;
                }
            }
        }

        function action_woocommerce_review_order_before_payment() {
            if (isset($this->moona_options['banner_above_cta_checkout_page']) && $this->moona_options['banner_above_cta_checkout_page'] === 'yes' && $this->is_moona_activated()) {
                if (isset($this->moona_options['_banner_above_cta_checkout_page_html'])) {
                    echo $this->moona_options['_banner_above_cta_checkout_page_html'];
                } else {
                    echo self::$banner_center_above_cta_html;
                }
            }
        }

        function action_woocommerce_mini_cart_contents() {
            if (isset($this->moona_options['banner_above_cta_minicart']) && $this->moona_options['banner_above_cta_minicart'] === 'yes' && $this->is_moona_activated()) {
                if (isset($this->moona_options['_banner_above_cta_minicart_html'])) {
                    echo $this->moona_options['_banner_above_cta_minicart_html'];
                } else {
                    echo self::$banner_center_above_cta_html;
                }
            }
        }

        function action_woocommerce_after_mini_cart() {
            if (isset($this->moona_options['banner_square_cart_page']) && $this->moona_options['banner_square_cart_page'] === 'yes' && $this->is_moona_activated()) {
                if (isset($this->moona_options['_banner_square_cart_page_html'])) {
                    echo $this->moona_options['_banner_square_cart_page_html'];
                } else {
                    echo self::$banner_square_full_img;
                }
            }
        }

        // Plugins loaded
        function wcmoona_plugin_to_load() {
            // i18n
            load_plugin_textdomain( 'discount-payment-moona', false, WCMOONA_BASE_NAME . '/languages/' );

            // Check if the class exist
            if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
                add_action( 'admin_notices', array( $this, 'wcmoona_woocommerce_fallback_notice' ) );
                return;
            }

            // This hook registers our PHP class as a WooCommerce payment gateway
            add_filter( 'woocommerce_payment_gateways', array( $this, 'add_moona_gateway_class' ) );
        }

        // Create order
        function action_create_moona_order( $order_id, $order ) {
            $this->amplitude_order_status($order_id, '', $order->get_status(), $order);
            // if (($order->get_payment_method() === 'discount-payment-moona' || $order->get_payment_method() === 'discount-payment-main-moona') &&
            //     ($order->get_status() === 'pending')) {
            // }
        }

        // Update order status
        // When the order status is updated (Pending payment - Failed - Processing - Completed - On hold - Canceled - Refunded)
        function action_order_status_changed( $order_id, $old_status, $new_status, $order ) {
            $this->amplitude_order_status($order_id, $old_status, $new_status, $order);
        }

        private function amplitude_order_status($order_id, $old_status, $new_status, $order) {
            // Amplitude event
            $shortVersion = explode(".", WC_VERSION);

            $paymentMethodID = $order->get_payment_method();
            if ($paymentMethodID === 'discount-payment-moona') {
                $paymentMethodID = 'moona-discount';
            } else if ($paymentMethodID === 'discount-payment-main-moona') {
                $paymentMethodID = 'moona-payment';
            }

            $eventProperties = array(
                'merchantSite' =>  get_bloginfo( 'name' ),
                'merchantURL' => get_permalink( wc_get_page_id( 'shop' ) ),
                'orderID' => $order_id,
                'currency' => get_woocommerce_currency(),
                'currencySymbol' => (htmlentities(get_woocommerce_currency_symbol()) === '&amp;pound;') ? '£' : get_woocommerce_currency_symbol(),
                'orderTotalValue' => $order->get_total(),
                'paymentMethodID' => $paymentMethodID,
                'paymentMethodLabel' => $order->get_payment_method_title(),
                'orderStatus' => $new_status, // $new_status OR $order->get_status()
                'orderStatusOld' => $old_status,
                'ecommerceSolution' => WCMOONA_ECOMMERCE_SOLUTION,
                'ecommerceVersion' => WC_VERSION, // ex : 5.0.0
                'ecommerceShortVersion' => $shortVersion[0].'.'.$shortVersion[1], // ex : 5.0
                'pluginVersion' => WCMOONA_PLUGIN_VERSION
            );

            if (isset($this->moona_options['merchant_id'])) {
                $eventProperties['merchantID'] = $this->moona_options['merchant_id'];
            }

            $moonaUserId = $order->get_meta('_moona_user_id');
            $GLOBALS['wc_moona_amplitude']->logEvent('Order - Status', $eventProperties, $moonaUserId);
        }

        // CTA Place order is clicked => order is created (pending)
        function action_checkout_order_processed( $order_id ) {
            $order = wc_get_order( $order_id );

            // Amplitude event
            $shortVersion = explode(".", WC_VERSION);

            $paymentKeys = array_keys( WC()->payment_gateways->get_available_payment_gateways() );
            $moonaDiscountPosition = array_search( 'discount-payment-moona',  $paymentKeys) ;
            $moonaPaymentPosition = array_search( 'discount-payment-main-moona', $paymentKeys) ;
            $moonaDiscountPosition = ($moonaDiscountPosition !== false) ? $moonaDiscountPosition : -1;
            $moonaPaymentPosition = ($moonaPaymentPosition !== false) ? $moonaPaymentPosition : -1;

            $paymentMethodID = $order->get_payment_method();
            if ($paymentMethodID === 'discount-payment-moona') {
                $paymentMethodID = 'moona-discount';
            } else if ($paymentMethodID === 'discount-payment-main-moona') {
                $paymentMethodID = 'moona-payment';
            }
            
            $eventProperties = array(
                'merchantSite' =>  get_bloginfo( 'name' ),
                'merchantURL' => get_permalink( wc_get_page_id( 'shop' ) ),
                'moonaPaymentPosition' => $moonaPaymentPosition,
                'moonaDiscountPosition' => $moonaDiscountPosition,
                'orderID' => $order_id,
                'currency' => get_woocommerce_currency(),
                'currencySymbol' => (htmlentities(get_woocommerce_currency_symbol()) === '&amp;pound;') ? '£' : get_woocommerce_currency_symbol(),
                'orderTotalValue' => $order->get_total(),
                'paymentMethodID' => $paymentMethodID,
                'paymentMethodLabel' => $order->get_payment_method_title(),
                'ecommerceSolution' => WCMOONA_ECOMMERCE_SOLUTION,
                'ecommerceVersion' => WC_VERSION, // ex : 5.0.0
                'ecommerceShortVersion' => $shortVersion[0].'.'.$shortVersion[1], // ex : 5.0
                'pluginVersion' => WCMOONA_PLUGIN_VERSION,
                'orderStatus' => $order->get_status() // Not mandatory because it will be always Pending
            );

            if (isset($this->moona_options['merchant_id'])) {
                $eventProperties['merchantID'] = $this->moona_options['merchant_id'];
            }

            $GLOBALS['wc_moona_amplitude']->logEvent('Click - Merchant checkout', $eventProperties);
        }

        // Checkout view
        function after_checkout_form_loaded( $checkout ){
            // print_r($checkout);
            // print_r(WC()->session);
            // print_r(get_current_user_id());
            // Only for connected user
            // print_r(wp_get_session_token());

            if (!$this->is_moona_activated()) {
                return false;
            }

            // TO BE RETRO COMPATIBLE : Save the merchant id
            $merchantId = null;
            if (!isset($this->moona_options['merchant_id'])) {

                $body = array(
                    'ecommerce_solution' => WCMOONA_ECOMMERCE_SOLUTION,
                    'ecommerce_version' => WC_VERSION,
                    'plugin_version' => WCMOONA_PLUGIN_VERSION,
                    'api_version' => WCMOONA_API_VERSION
                );

                $url = (( $this->moona_options['test_mode'] === 'yes' ) ? MOONA_API_STAGING_ENDPOINT : MOONA_API_ENDPOINT);
                $url .= 'ecommerce/merchant';

                $response = wp_remote_post( $url, array(
                                    'method' => 'POST',
                                    'timeout' => 20,
                                    'headers' => array(
                                        'Content-Type' => 'application/json; charset=utf-8',
                                        'x-api-key' => ( $this->moona_options['test_mode'] === 'yes' ) ? $this->moona_options['moona_sk_test'] : $this->moona_options['moona_sk_live']
                                    ),
                                    'body' => json_encode($body)
                                )
                            );

                if ( is_wp_error( $response ) || wp_remote_retrieve_response_code( $response ) !== 200 ) {
                    $this->log( 'error on customize_banners() process : ' . print_r($response, true) );
                } else {
                    $body = json_decode( $response['body'], true );
                    $options = $this->moona_options;
                    $options['merchant_id'] = $body['merchantId'];
                    update_option('woocommerce_discount-payment-moona_settings', $options);
                    $merchantId = $options['merchant_id'];
                }
            } else {
                $merchantId = $this->moona_options['merchant_id'];
            }

            // Amplitude event
            $shortVersion = explode(".", WC_VERSION);
            
            $paymentKeys = array_keys( WC()->payment_gateways->get_available_payment_gateways() );
            $moonaDiscountPosition = array_search( 'discount-payment-moona',  $paymentKeys) ;
            $moonaPaymentPosition = array_search( 'discount-payment-main-moona', $paymentKeys) ;
            $moonaDiscountPosition = ($moonaDiscountPosition !== false) ? $moonaDiscountPosition : -1;
            $moonaPaymentPosition = ($moonaPaymentPosition !== false) ? $moonaPaymentPosition : -1;

            $eventProperties = array(
                'merchantSite' => get_bloginfo( 'name' ),
                'merchantURL' => get_permalink( wc_get_page_id( 'shop' ) ),
                'moonaPaymentPosition' => $moonaPaymentPosition,
                'moonaDiscountPosition' => $moonaDiscountPosition,
                'currency' => get_woocommerce_currency(),
                'currencySymbol' => (htmlentities(get_woocommerce_currency_symbol()) === '&amp;pound;') ? '£' : get_woocommerce_currency_symbol(),
                'orderTotalValue' => WC()->cart->total,
                'ecommerceSolution' => WCMOONA_ECOMMERCE_SOLUTION,
                'ecommerceVersion' => WC_VERSION, // ex : 5.0.0
                'ecommerceShortVersion' => $shortVersion[0].'.'.$shortVersion[1], // ex : 5.0
                'pluginVersion' => WCMOONA_PLUGIN_VERSION
            );

            if (isset($merchantId)) {
                $eventProperties['merchantID'] = $merchantId;
            }

            $GLOBALS['wc_moona_amplitude']->logEvent('View - Merchant checkout', $eventProperties);
        }

        /**
         * Add settings link
         */
        function add_action_settings_link( $links ){
            $action_links = array(
                'settings' => '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout&section=discount-payment-moona' ) . '" aria-label="' . esc_attr__( 'View Moona settings', 'discount-payment-moona' ) . '">' . esc_html__( 'Settings', 'discount-payment-moona' ) . '</a>',
            );
            return array_merge( $action_links, $links );
        }

        /**
         * Add notice
         */
        function wcmoona_woocommerce_fallback_notice()
        {
            $html = '<div class="notice notice-error">';
            $html .= '<p>' . __( 'The WooCommerce Moona plugin requires the latest version of <a href="http://wordpress.org/extend/plugins/woocommerce/" target="_blank">WooCommerce</a> to work!', 'discount-payment-moona' ) . '</p>';
            $html .= '</div>';
            echo  $html ;
        }

        /**
         * Add Moona gateway
         */
        function add_moona_gateway_class( $methods )
        {
            require_once WCMOONA_BASE_PATH . '/includes/class_wc_moona_gateway.php';
            $methods[] = 'WC_Moona_Gateway';

            require_once WCMOONA_BASE_PATH . '/includes/class_wc_moona_main_gateway.php';
            $methods[] = 'WC_Moona_Main_Gateway';

            return $methods;
        }

        /**
         * Logging method.
         *
         * @param string $message Log message.
         * @param string $level   Optional. Default 'alert'.
         *     emergency|alert|critical|error|warning|notice|info|debug
         */
        public static function log( $message, $level = 'alert' )
        {
            if ( ! class_exists( 'WC_Logger' ) ) {
                return;
            }

            if ( empty(self::$log) ) {
                self::$log = wc_get_logger();
            }
            self::$log->log( $level, $message, array(
                'source' => 'moona',
            ) );
        }

        /******************************************************************************/
        /*********************** CRON PENDING ORDERS ** *******************************/
        /******************************************************************************/

        /**
         * Add new 'every 10 minutes' schedule
         * @param $schedules
         * @return mixed
         */
        public function cron_every_ten_minutes( $schedules ) {
            // Add once ten_minutes to the existing schedules.
            if ( !isset($schedules["ten_minutes"]) ) {
                $schedules['ten_minutes'] = array(
                    'interval' => 600,
                    'display' => __( 'Every 10 minutes' )
                );
            }
            return $schedules;
        }

        /**
         * Create cron jobs to clear pending_orders
         */
        public function create_moona_cron_jobs() {
            wp_clear_scheduled_hook( 'moona_cancel_pending_orders' );

            if (!wp_next_scheduled('moona_cron')) {
                wp_schedule_event(time(), 'ten_minutes', 'moona_cron');
            }
        }

        /**
         * Cancel abandoned moona payment after 10 minutes
         */
        public function update_orders() {
            $orders = $this->get_pending_moona_orders();

            // Only pending orders
            foreach ($orders as $order) {
                $isPaid = $order->get_meta('_is_paid');

                // Add a note
                if ( $isPaid === 'true') {
                    $order->add_order_note( __( 'Order is paid but your server seems to stop the communication with Stripe. Please contact us through Moona with the order id (merchant@moona.com).', 'discount-payment-moona'), false );
                    $order->update_status( 'on-hold', true );
                    // TODO : Later, check to refresh directly the IPN here
                } else {
                    wp_trash_post($order->get_id());
                }
            }
        }

        /**
         * Get created via moona orders
         * @return array
         */
        public function get_pending_moona_orders() {
            // Get 'pending payment' orders created via moona since more than 10 min
            $args = array(
                'limit' => -1,
                'type' => 'shop_order',
                // 'created_via' => 'moona-payment',
                'payment_method' => 'discount-payment-main-moona',
                'status' => array( 'pending' ),
                'date_created' => '<' . ( time() - 600 )
            );
            $ordersMoonaPayment = wc_get_orders( $args );

            // $args['created_via'] = 'moona-discount';
            $args['payment_method'] = 'discount-payment-moona';
            $ordersMoonaDiscount = wc_get_orders( $args );
            $results = array_merge($ordersMoonaPayment, $ordersMoonaDiscount);

            return $results;
        }
    }
    /******************************************************************************/
    /********************** ACTIVATION/DEACTIVATION *******************************/
    /******************************************************************************/

    function wcmoona_plugin_status($status) {

        $body = array(
            'ecommerce_solution' => WCMOONA_ECOMMERCE_SOLUTION,
            'ecommerce_version' => ( defined( 'WC_VERSION' ) ) ? WC_VERSION : '',
            'plugin_version' => WCMOONA_PLUGIN_VERSION,
            'api_version' => WCMOONA_API_VERSION,
            'status' => $status,
            'device_id' => wp_get_session_token()
        );

        $headers = array( 'Content-Type' => 'application/json; charset=utf-8' );

        $moona_options = get_option('woocommerce_discount-payment-moona_settings');
        if ($moona_options && ($status === 'enabled' || $status === 'disabled' || $status === 'updated')) {

            $body['options'] = json_encode($moona_options);

            $paymentKeys = array_keys( WC()->payment_gateways->get_available_payment_gateways() );
            $moonaDiscountPosition = array_search( 'discount-payment-moona',  $paymentKeys) ;
            $moonaPaymentPosition = array_search( 'discount-payment-main-moona', $paymentKeys) ;
            $moonaDiscountPosition = ($moonaDiscountPosition !== false) ? $moonaDiscountPosition : -1;
            $moonaPaymentPosition = ($moonaPaymentPosition !== false) ? $moonaPaymentPosition : -1;

            $body['payment_position'] = ($moonaDiscountPosition !== false) ? $moonaDiscountPosition : -1;
            $body['payment_position_main'] = ($moonaPaymentPosition !== false) ? $moonaPaymentPosition : -1;

            // NOTE : CHANGE BY PROD ON RELEASE
            $headers['x-api-key'] = $moona_options['moona_sk_live']; // moona_sk_live OR moona_sk_test
        }
        else {
            $body['merchant_email'] = (get_option('admin_email')) ? get_option('admin_email') : '';
            $body['merchant_fullname'] = (get_option('blogname')) ? get_option('blogname') : '';
            $body['merchant_website'] = (get_site_url()) ? get_site_url() : ''; // get_option('home') or get_option('siteurl')
        }

        // NOTE : CHANGE BY PROD ON RELEASE
        $url = MOONA_API_ENDPOINT; // MOONA_API_ENDPOINT OR MOONA_API_STAGING_ENDPOINT
        $url .= 'ecommerce/status';
        // error_log(print_r('wcmoona_plugin_activated process : ' . json_encode($body), true));

        $response = wp_remote_post( $url, array(
                'method' => 'POST',
                'timeout' => 20,
                'headers' => $headers,
                'body' => json_encode($body)
            )
        );

        if ( is_wp_error( $response ) || wp_remote_retrieve_response_code( $response ) != 200 ) {
            error_log(print_r('error on wcmoona_plugin_status()' . $status . ' process : ' . print_r($response, true), true));
            return $response;
        }
    }

    /**
     * No x-api-key yet
     * Status : installed
     */
    function wcmoona_plugin_activated() {
        wcmoona_plugin_status('installed');
    }

    /**
     * No x-api-key yet
     * Status : uninstalled
     */
    function wcmoona_plugin_deactivated() {
        wcmoona_plugin_status('uninstalled');
    }

    /**
     * No x-api-key yet
     * Status : uninstalled
     */
    function wcmoona_plugin_uninstall() {
        wcmoona_plugin_status('uninstalled');
    }

    // register_activation_hook( __FILE__,  array( $this, 'wcmoona_plugin_activated'), 0 );
    register_activation_hook( __FILE__,  'wcmoona_plugin_activated');
    register_deactivation_hook( __FILE__,  'wcmoona_plugin_deactivated');
    register_uninstall_hook( __FILE__,  'wcmoona_plugin_uninstall');

    /**
     * On Moona upgrade
     */
    function upgrade_moona_plugin( $upgrader_object, $options ) {
        // The path to our plugin's main file
        $moona_plugin = plugin_basename( __FILE__ );
        $moona_updated = false;

        // If an update has taken place and the updated type is plugins and the plugins element exists
        if( $options['action'] == 'update' && $options['type'] == 'plugin' && isset( $options['plugins'] ) ) {
            // Iterate through the plugins being updated and check if ours is there
            foreach( $options['plugins'] as $plugin ) {
                if( $plugin == $moona_plugin ) {

                    $moona_updated = true;

                    // Set a transient to record that our plugin has just been updated
                    // set_transient( 'wp_upe_updated', 1 );
                    break;

                    // Check the transient to see if we've just updated the plugin
                    // if( get_transient( 'wp_upe_updated' ) ) {
                    //     echo '<div class="notice notice-success">' . __( 'Thanks for updating', 'wp-upe' ) . '</div>';
                    //     delete_transient( 'wp_upe_updated' );
                    // }
                }
            }
        }

        if ( ! $moona_updated ) {
            return;
        }

        // Send the update info
        wcmoona_plugin_status('updated');
    }

    add_action( 'upgrader_process_complete', 'upgrade_moona_plugin',10, 2);
}

// Instantiate the plugin class and add it to the set of globals
$GLOBALS['wc_moona'] = WC_Moona::get_instance();
